unit Restruct;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB;

type
    TDoRestruct   = function (winHandle: Integer; wdir, pdir: pChar): Integer;
    TDoRestruct_1 = function (winHandle: Integer; wdir, pdir, cmdTable: pChar): Integer;
    TDoRestruct_2 = function (winHandle: Integer; wdir, pdir, cmdTable, defTable: pChar): Integer;
    TRename_ErrTable = function (tName: pChar): Integer;
    TRename_ChgTable = function (tName: pChar): Integer;
    TRename_CmdTable = function (tName: pChar): Integer;
    TRename_DefTable = function (tName: pChar): Integer;
    TEnumStructure_Print = function (winHandle: Integer; wdir, pdir, tName, outFile: pChar): Integer;
    TEnumStructure_Store = function (winHandle: Integer; wdir, pdir, tName, cmdTable: pChar): Integer;
    TIsDemo = function : Integer;

  TRESTRUCTURE = class(TComponent)
  private
    { Private declarations }
    FhDll     : THandle;
    FcmdTable : String;
    FdefTable : String;
    FworkDir  : String;
    FprivDir  : String;
    FerrTable : String;
    FchgTable : String;
    savedPrivDir : String;

    FerrCode : Integer;
    FwDir    : pChar;
    FpDir    : pChar;
    FmTable  : pChar;
    FdTable  : pChar;

    FclientHandle : Hwnd;

   PDoRestruct   : TDoRestruct;
   PDoRestruct_1 : TDoRestruct_1;
   PDoRestruct_2 : TDoRestruct_2;
   PRename_ErrTable : TRename_ErrTable;
   PRename_ChgTable : TRename_ChgTable;
   PRename_CmdTable : TRename_CmdTable;
   PRename_DefTable : TRename_DefTable;
   PEnumStructure_Print : TEnumStructure_Print;
   PEnumStructure_Store : TEnumStructure_Store;
   PisDemo : TisDemo;

   procedure loaded ; override;
   procedure Open ;
   procedure close ;
   procedure closeNames;
   procedure openNames ;

  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create (Aowner : TComponent); override;
    destructor  Destroy ; override;
    procedure DoRestruct ;
    procedure DoRestruct_1 (cTab: String);
    procedure DoRestruct_2 (cTab, dTab: String);
    procedure EnumStructure_print (sTab, dest: String);
    procedure EnumStructure_store (sTab, dest: String);

    procedure Rename_errTable (tableName : String);
    procedure Rename_chgTable (tableName : String);
    procedure Rename_defTable (tableName : String);
    procedure Rename_cmdTable (tableName : String);

  published
    { Published declarations }
     property cmdTable : String read FcmdTable write Fcmdtable stored True;
     property defTable : String read FdefTable write Fdeftable stored True;
     property chgTable : String read FchgTable write FchgTable stored TRUE;
     property errTable : String read FerrTable write FerrTable stored TRUE;
     property clientHandle : Hwnd read FclientHandle write FclientHandle default 0;
     property workDir  : String read FworkDir write FworkDir stored True;
     property privDir  : String read FprivDir write FprivDir stored TRUE;
     property errCode : Integer read FerrCode write FerrCode default 0;
{     property isDemo : Integer;}
   Public
     isDemo : Integer;
  end;

procedure Register;

implementation

constructor TRESTRUCTURE.Create (AOwner : TComponent);
begin
     inherited Create(AOwner);
     cmdTable := 'modstrct.db';
     deftable := 'temp.db';
     privDir := session.privateDir;

     FclientHandle := 0;
     open;
     if Length(cmdTable) > 0 then rename_CmdTable(cmdTable);
     if Length(defTable) > 0 then rename_DefTable(defTable);
     if Length(chgTable) > 0 then rename_ChgTable(chgTable);
     if Length(errTable) > 0 then rename_ErrTable(errTable);

     isDemo := PIsDemo;

end;

destructor TRESTRUCTURE.Destroy;
begin
     close;
     inherited Destroy;
end;

procedure TRESTRUCTURE.Loaded ;
begin
     inherited Loaded;
     if Length(cmdTable) = 0 then cmdTable := 'modstrct.db';
     if Length(defTable) = 0 then deftable := 'temp.db';
     if Length (privDir) = 0 then privDir := session.privateDir;

     if Length(cmdTable) > 0 then rename_CmdTable(cmdTable);
     if Length(defTable) > 0 then rename_DefTable(defTable);
     if Length(chgTable) > 0 then rename_ChgTable(chgTable);
     if Length(errTable) > 0 then rename_ErrTable(errTable);
end;

procedure TRESTRUCTURE.DoRestruct ;
begin
     if @PDoRestruct <> nil then
        begin
        {FerrCode := PDoRestruct (parent.ClientHandle, wDir, pDir);}
        openNames;
        FerrCode := PDoRestruct (FClientHandle, FwDir, FpDir);
        closeNames;
        end;
end;

procedure TRESTRUCTURE.DoRestruct_1 (cTab: String) ;
var
   FcTab: PChar;
begin
     if @PDoRestruct <> nil then
        begin
        {FerrCode := PDoRestruct (parent.ClientHandle, wDir, pDir);}
        openNames;
        FcTab := StrAlloc(length(cTab) + 1);
        StrPCopy (FcTab, cTab);
        FerrCode := PDoRestruct_1 (FClientHandle, FwDir, FpDir, FcTab);
        StrDispose (FcTab);
        closeNames;
        end;
end;

procedure TRESTRUCTURE.DoRestruct_2 (cTab, dTab: String) ;
var
   FcTab, FdTab: PChar;
   temp : string;
begin
     if @PDoRestruct <> nil then
        begin
        {FerrCode := PDoRestruct (parent.ClientHandle, wDir, pDir);}
        openNames;
        FcTab := StrAlloc(length(cTab) + 1);
        StrPCopy (FcTab, cTab);
        FdTab := StrAlloc(length(dTab) + 1);
        StrPCopy (FdTab, dTab);
        FerrCode := PDoRestruct_2 (FClientHandle, FwDir, FpDir, FcTab, FdTab);
        str(FerrCode, temp);
        StrDispose (FcTab);
        StrDispose (FdTab);
        closeNames;
        end;
end;

procedure TRESTRUCTURE.EnumStructure_print ( sTab, dest: String);
var
   psTab, pdest: pChar;
begin
     if @PEnumStructure_print <> nil then
        begin
        openNames;

        psTab := StrAlloc(Length(stab) + 1);
        StrPCopy (psTab, sTab);
        pdest := StrAlloc(Length(dest) + 1);
        StrPCopy (pdest, dest);

        FerrCode := PEnumStructure_print (FClientHandle, FwDir, FpDir, psTab, pdest);
        StrDispose (psTab);
        StrDispose (pdest);

        closeNames;

        end;
end;

procedure TRESTRUCTURE.EnumStructure_store ( sTab, dest: String);
var
   psTab, pdest: pChar;
begin
     if @PEnumStructure_print <> nil then
        begin
        openNames;

        psTab := StrAlloc(Length(stab) + 1);
        StrPCopy (psTab, sTab);
        pdest := StrAlloc(Length(dest) + 1);
        StrPCopy (pdest, dest);

        FerrCode := PEnumStructure_store (FClientHandle, FwDir, FpDir, psTab, pdest);
        StrDispose (psTab);
        StrDispose (pdest);

        closeNames;

        end;
end;

procedure TRESTRUCTURE.open ;
begin
     if FhDll = 0 then
          FhDll := LoadLibrary('restruct.dll');

     {Get the Procedure addresses}
     @PDoRestruct := GetProcAddress(FhDll, 'DoRestruct');
     @PDoRestruct_1 := GetProcAddress(FhDll, 'DoRestruct_1');
     @PDoRestruct_2 := GetProcAddress(FhDll, 'DoRestruct_2');
     @Prename_ErrTable := GetProcAddress(FhDll, 'rename_ErrTable');
     @Prename_ChgTable := GetProcAddress(FhDll, 'rename_ChgTable');
     @Prename_CmdTable := GetProcAddress(FhDll, 'rename_CmdTable');
     @Prename_DefTable := GetProcAddress(FhDll, 'rename_DefTable');
     @PenumStructure_print := GetProcAddress(FhDll, 'enumStructure_print');
     @PenumStructure_store := GetProcAddress(FhDll, 'enumStructure_store');
     @PisDemo := GetProcAddress(FhDll, 'isRestructDemo');
end;

procedure TRESTRUCTURE.close ;
begin
     {ShowMessage (who + ' caused close of Restruct.dll ' + IntToStr(hDll));}
     if FhDll <> 0
          then FreeLibrary(FhDll);
     FhDll := 0;
     @PDoRestruct := nil;
     @PDoRestruct_1 := nil;
     @PDoRestruct_2 := nil;
     @Prename_ErrTable := nil;
     @Prename_ChgTable := nil;
     @Prename_CmdTable := nil;
     @Prename_DefTable := nil;
     @PenumStructure_print := nil;
     @PenumStructure_store := nil;
     @PisDemo := nil;

end;

procedure TRESTRUCTURE.openNames ;
begin
  {get the private Directory pChar set up}
     FpDir := StrAlloc(Length(FprivDir) + 1);
     StrPCopy (FpDir, FprivDir);
     {save away old session.privateDir and set to current value of privDir}
     {// removed 7/20/95
     savedPrivDir := session.PrivateDir;
     session.PrivateDir := FprivDir;
     }
  {get the work Directory pChar set up}
     FwDir := StrAlloc(length(FworkDir) +1);
     StrPCopy (FwDir, FworkDir);

  {get the defTable pChar set up}
     FdTable := StrAlloc(length(defTable) +1);
     StrPCopy (Fdtable, defTable);

  {get the modfTable pChar set up}
     FmTable := StrAlloc(length(cmdTable) +1);
     StrPCopy (Fmtable, cmdTable);
end;

procedure TRESTRUCTURE.closeNames;
Begin
  {release the allocated pChar's}
     StrDispose (FwDir);
     StrDispose (FpDir);
     StrDispose (Fdtable);
     StrDispose (Fmtable);
     {removed 7/20/95
     session.PrivateDir := savedPrivDir;
     }
end;

procedure TRESTRUCTURE.Rename_ErrTable (tableName : String);
var
   temp : pChar;
Begin
     temp := StrAlloc(length(tableName) +1);
     StrPCopy (temp, tableName);
     FerrCode := PRename_ErrTable(temp);
     if FerrCode = 0 then errTable := tableName;
     StrDispose (temp);
end;

procedure TRESTRUCTURE.Rename_ChgTable (tableName : String);
var
   temp : pChar;
Begin
     temp := StrAlloc(length(tableName) +1);
     StrPCopy (temp, tableName);
     FerrCode := PRename_ChgTable(temp);
     if FerrCode = 0 then chgTable := tableName;
     StrDispose (temp);
end;
procedure TRESTRUCTURE.Rename_DefTable (tableName : String);
var
   temp : pChar;
Begin
     temp := StrAlloc(length(tableName) +1);
     StrPCopy (temp, tableName);
     FerrCode := PRename_DefTable(temp);
     if FerrCode = 0 then defTable := tableName;
     StrDispose (temp);
end;
procedure TRESTRUCTURE.Rename_CmdTable (tableName : String);
var
   temp : pChar;
Begin
     temp := StrAlloc(length(tableName) +1);
     StrPCopy (temp, tableName);
     FerrCode := PRename_CmdTable(temp);
     if FerrCode = 0 then cmdTable := tableName;
     StrDispose (temp);
end;
procedure Register;
begin
  RegisterComponents('Data Access', [TRESTRUCTURE]);
end;

end.
